/*jslint browser: true*/
/*global define*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!./alignmentdialog-test.html',
    'jquery',
    'de_epages/design/ui/alignmentdialog',
    '$ready!'
], function (QUnit, fixture, $, de_epages) {

    'use strict';

    QUnit.module('de_epages/design/ui/alignmentdialog', {
        setup: function () {
            fixture.init();
            // Create (fake dojo) widget.
            window.myFakeDojoWidget = {
                alignmentNode: $('#FakeDojoInputs input[name="17507:Alignment"]').get(0),
                topNode: $('#FakeDojoInputs input[name="17507:Top"]').get(0),
                leftNode: $('#FakeDojoInputs input[name="17507:Left"]').get(0),
                contentNode: $('#MBOStorefront .NavigationElementWidget .dojo-contentNode').get(0),
                orientation: 'LeftRight',
                navbarNode: function () {
                    return $('#NavBar2777').get(0);
                },
                siblingWidgets: function () {
                    return [this];
                }
            };

            // Create test widget.
            $('#qunit-fixture .AlignNode').designUiAlignmentdialog({
                dojoWidget: window.myFakeDojoWidget,
                getNavBarContainers: function (doc) {
                    return $('.HorizontalNavBar');
                },
                top$: $,
                topDocument: document
            });

            // Check indication of selected element
            QUnit.strictEqual($('#MBOStorefront .NavigationElementWidget .dojo-contentNode')
                .first()
                .children('[id^="Element"]')
                .children()
                .not('script')
                .first()
                .hasClass('de_epages-designUiAlignmentdialog-selectedItem'),
                true,
                '*home link* NavigationElement is selected');
        },
        teardown: function () {
            fixture.destroy();

            $('#qunit-fixture .AlignNode').designUiAlignmentdialog('destroy');

            QUnit.strictEqual($('#qunit-fixture .AlignNode').is(':data("ui-designUiAlignmentdialog")'),
                false,
                '*destroy* method removed data attribute *ui-designUiAlignmentdialog*');
        }
    });

    QUnit.test("prototype methods", 8 + 2, function () {
        // Check prototype methods.
        $.each(['_create', '_setLeftAlign', '_setRightAlign', '_setPosition', '_shortcuts', '_toggleAlign', '_fireChange', 'destroy'], function (i, fn) {

            QUnit.strictEqual($.isFunction($.ui.designUiAlignmentdialog.prototype[fn]),
                true,
                '$.isFunction(' + fn + ')');

        });
    });

    QUnit.test("ui: positioning with arrow buttons", 6 + 2, function () {
        // Test *up* arrow.
        QUnit.triggerEvent($('.fa-caret-up').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Top"]').val(),
            '-1',
            'Change triggered on *input[name="17507:Top"]* by clicking *up* arrow');

        // Test *down* arrow.
        QUnit.triggerEvent($('.fa-caret-down').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Top"]').val(),
            '0',
            'Change triggered on *input[name="17507:Top"]* by clicking *down* arrow');

        // Test *left* arrow.
        QUnit.triggerEvent($('.fa-caret-left').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Left"]').val(),
            '-1',
            'Change triggered on *input[name="17507:Left"]* by clicking *left* arrow');

        // Test *right* arrow.
        QUnit.triggerEvent($('.fa-caret-right').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Left"]').val(),
            '0',
            'Change triggered on *input[name="17507:Left"]* by clicking *left* arrow');

        // Test *reset* button.
        QUnit.triggerEvent($('.ep-uiSimpledialog div.CommandLinkSmall').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Left"]').val(),
            'auto',
            'Change triggered on *input[name="17507:Left"]* by clicking *reset* button');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Top"]').val(),
            'auto',
            'Change triggered on *input[name="17507:Top"]* by clicking *reset* button');
    });

    QUnit.test("ui: positioning with keyboard", 4 + 2, function () {
        // Test *left* key shortcut.
        $('body').triggerHandler($.Event('keydown', {
            which: $.ui.keyCode.LEFT
        }));

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Left"]').val(),
            '-2',
            'Change triggered on *input[name="17507:Left"]* by pressing *left* arrow');

        // Test *right* key shortcut.
        $('body').triggerHandler($.Event('keydown', {
            which: $.ui.keyCode.RIGHT
        }));

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Left"]').val(),
            '0',
            'Change triggered on *input[name="17507:Left"]* by pressing *right* arrow');

        // Test *down* key shortcut.
        $('body').triggerHandler($.Event('keydown', {
            which: $.ui.keyCode.DOWN
        }));

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Top"]').val(),
            '2',
            'Change triggered on *input[name="17507:Top"]* by pressing *down* arrow');

        // Test *up* key shortcut.
        $('body').triggerHandler($.Event('keydown', {
            which: $.ui.keyCode.UP
        }));

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Top"]').val(),
            '0',
            'Change triggered on *input[name="17507:Top"]* by pressing *up* arrow');
    });

    QUnit.test("ui: alignment", 9 + 2, function () {

        // Click *align right* on an element that is already in a navbar
        // which has all its elements aligned to the right. => Nothing happens.
        QUnit.strictEqual($('select[name$="' + 'Roof' + 'Align"]').val(),
            'right',
            'NavBar Roof is aligned *right*');

        QUnit.triggerEvent($('.ep-uiSimpledialog .fa-step-forward').get(0), 'click');

        QUnit.strictEqual($('select[name$="' + 'Roof' + 'Align"]').val(),
            'right',
            'NavBar Roof is *still* aligned *right*, after clicking *align right* on the *dialog*');

        QUnit.triggerEvent($('.ep-uiSimpledialog .fa-step-backward').get(0), 'click');

        // This time, click *align left*.
        QUnit.strictEqual($('select[name$="' + 'Roof' + 'Align"]').val(),
            'custom',
            'Alignment of NavBar Roof changed to *custom* after aligning one of its navelements individually');

        // Now align the element back to the right.
        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Alignment"]').val(),
            'Left',
            '*home link* NavigationElement has alignment *left* (default)');

        QUnit.triggerEvent($('.ep-uiSimpledialog .fa-step-forward').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Alignment"]').val(),
            'Right',
            '*home link* NavigationElement has alignment *right*, after clicking *align right* on the *dialog*');

        // Ok, so now let's go back to the left.
        QUnit.triggerEvent($('.ep-uiSimpledialog .fa-step-backward').get(0), 'click');

        QUnit.strictEqual($('#FakeDojoInputs input[name="17507:Alignment"]').val(),
            'Left',
            '*home link* NavigationElement has alignment *left*, after clicking *align left* on the *dialog*');

        // Test visibilty of alignment buttons in *dialog*
        QUnit.strictEqual($('.ep-uiSimpledialog ul li:first-child')
            .first()
            .is(':visible'),
            true,
            'alignment buttons are visible in *horizontal* navbar');

        // Close the dialog by clicking on a navbar.
        QUnit.triggerEvent($('.NavBarRoof').get(0), 'click');

        QUnit.strictEqual($('#MBOStorefront .NavigationElementWidget .dojo-contentNode')
            .first()
            .children('[id^="Element"]')
            .children().not('script')
            .first()
            .hasClass('de_epages-designUiAlignmentdialog-selectedItem'),
            false,
            '*home link* NavigationElement is not selected after closing the dialog');

        // Test visibilty of alignment buttons in *dialog*
        // This time as if we were inside a *vertical* navbar.
        window.myFakeDojoWidget.orientation = "UpDown";

        $('#qunit-fixture .AlignNode').designUiAlignmentdialog('option', 'dojoWidget', window.myFakeDojoWidget);

        QUnit.triggerEvent($('#qunit-fixture .AlignNode').get(0), 'click');

        QUnit.strictEqual($('.ep-uiSimpledialog ul li:first-child').first().is(':visible'), false, 'alignment buttons are *not* visible in *vertical* navbar');
    });
});
