/*global define, google*/
define('de_epages/externalcontent/gadgetdialog/maps-view', [
    'jquery',
    'ep',
    './constants',
    './config-view',
    '$tmpl!./maps-view',
    '$dict!../dictionary',

    '../ui/maps' // The gadget to configure.
], function ($, ep, CONSTANTS, ConfigView, template, dict) {

    'use strict';

    return ConfigView.extend({

        alias: 'maps',

        events: ConfigView.events,

        dict: dict,

        template: template,

        onApply: function () {
            // VALIDATION
            // https://jira.epages.de/browse/EPG-26271
            var invalidEntries = this.$('form').find('input:invalid'),
                $form;

            if (invalidEntries.length) {
                if (!this.fakesubmit) {
                    $form = this.$('form');
                    this.fakesubmit = $('<button type="submit">').addClass('HideElement').attr('form', $form.attr('id')).appendTo($form);
                }
                this.fakesubmit.trigger('click');
                return false;
            }

            return true;
        },

        onRadio: function (e) {
            var target = e.target,
                self = this;

            if (target.value !== self.gOptions[target.name]) {
                if (target.value !== 'klicktel') {
//#JSCOVERAGE_IF 0
                    // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
                    // show fieldset with name *target.value*, hide the others.
                    self.gOptions = {};
                    self.gOptions[target.name] = target.value;
                    self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
                    self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
                    // Set remaining *self.gOptions* according to fieldset shown.
                    self.$el.find('fieldset[name="' + target.value + '"] :input').not('[name="address"]').filter(function () {
                        return this.value !== '';
                    }).trigger('change', true);
                    self.renderPreview();
//#JSCOVERAGE_ENDIF
                } else {
                    // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
                    // show fieldset with name *target.value*, hide the others.
                    self.gOptions = {};
                    self.gOptions[target.name] = target.value;
                    self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
                    self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
                    // Set remaining *self.gOptions* according to fieldset shown.
                    self.$el.find('fieldset[name="' + target.value + '"] :input').trigger('change', true);
                    self.renderPreview();
                }
            }
        },

        addressToLatLng: function (address, noPreview) {
            var self = this;

            self.geocoder.geocode({
                address: address
            }, $.proxy(function (result, status) {
                var self = this;
                if (status === 'OK' && result && result[0] && result[0].geometry && result[0].geometry.location) {
                    delete self.gOptions.address;
                    delete self.gOptions.center;
                    self.setOptions({
                        lat: result[0].geometry.location.lat(),
                        lng: result[0].geometry.location.lng(),
                        address: address
                    });
                    if (!noPreview) {
                        self.renderPreview();
                    }
                }
            }, self));
        },

        renderPreview: function () {
            var self = this,
                preview = self.elements.preview,
                parent = preview.parent(),
                gadgetPath = CONSTANTS.gWidgetNamespace + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
            if (preview.data('de_epages' + '-' + gadgetPath)) {
                preview[gadgetPath]('destroy');
            }
            preview.remove();
            preview = self.elements.preview = $('<div class="PreviewNode">').appendTo(parent);
            preview[gadgetPath]($.extend({}, self.gOptions, {
                renderMapsPreview: true,
                callback: function (type, map, marker) {
                    self.map = map;
                    self.marker = marker;
                    switch (type) {
                    case 'google':
                        marker.setDraggable(true);
                        google.maps.event.addListener(marker, 'dragend', $.proxy(function () {
                            var position = self.marker.getPosition(),
                                center = self.map.getCenter();

                            self.setOptions({
                                lat: position.lat(),
                                lng: position.lng(),
                                center: {
                                    lat: center.lat(),
                                    lng: center.lng()
                                }
                            });

                        }, self));

                        google.maps.event.addListener(map, 'dragend', $.proxy(function () {
                            var center = self.map.getCenter();

                            self.setOptions({
                                center: {
                                    lat: center.lat(),
                                    lng: center.lng()
                                }
                            });

                        }, self));

                        google.maps.event.addListener(map, 'zoom_changed', $.proxy(function () {
                            self.setOptions({
                                zoom: self.map.getZoom()
                            });

                        }, self));

                        google.maps.event.addListener(map, 'maptypeid_changed', $.proxy(function () {
                            self.setOptions({
                                mapType: self.map.getMapTypeId()
                            });

                        }, self));

                        break;
//#JSCOVERAGE_IF 0
                    case 'bing':
                        self.map.AttachEvent('onchangeview', $.proxy(function (e) {
                            var center = this.map.GetCenter();

                            this.setOptions({
                                mapType: e.mapStyle,
                                zoom: e.zoomLevel,
                                center: {
                                    lat: center.Latitude,
                                    lng: center.Longitude
                                }
                            });

                        }, self));
//#JSCOVERAGE_ENDIF
                        break;
                    }
                }
            }));
        },

        render: function () {
            var self = this,
                renderedTemplate,
                types = [],
                defaults,
                i;

            if (self.differTypes) {
                // Merge *self.gOptions* in *types* for templating. (See below.)
                for (i = 0; i < self.gDefaults.length; i = i + 1) {
                    defaults = self.gDefaults[i];
                    types.push(defaults.type === self.gOptions.type ? self.gOptions : defaults);
                }
            }

            renderedTemplate = self.template(self.differTypes ? {
                'types': types,
                'gOptions': self.gOptions
            } : self.gOptions).dictParse(dict, true);
            renderedTemplate.find(':input').each(function () {
                if (this.placeholder) {
                    this.placeholder = dict.translate(this.placeholder);
                }
                $(this).uiInput();
            });
            self.$el.append(renderedTemplate);
            self.elements = renderedTemplate.tmplItem('elements'); // preview element z.B.

            ep.ajax({
                url: 'https://www.google.com/jsapi',
                dataType: 'script',
                cache: true // We would not want to load the same *script* twice.
            }).done(function () {
                google.load('maps', '3', {
                    other_params: 'libraries=places&sensor=false',
                    callback: function () {
                        self.geocoder = new google.maps.Geocoder();
                        self.$('input[name="address"]').each(function () {
                            var autocomplete = new google.maps.places.Autocomplete(this);
                            google.maps.event.addListener(autocomplete, 'place_changed', function () {
//#JSCOVERAGE_IF 0
                                var place = this.getPlace();
                                if (place.geometry && place.geometry.location) {
                                    delete self.gOptions.address;
                                    delete self.gOptions.center;
                                    self.setOptions({
                                        lat: place.geometry.location.lat(),
                                        lng: place.geometry.location.lng()
                                    });
                                    self.renderPreview();
                                }
                                self.addressToLatLng(place.formatted_address);
                            });
//#JSCOVERAGE_ENDIF
                        });
                        if (!self.gOptions.lat || !self.gOptions.lng) {
                            if (ep.config.shopAddress) {
                                self.addressToLatLng(ep.config.shopAddress, true);
                                self.$('input[name="address"]').each(function () {
                                    this.value = ep.config.shopAddress;
                                });
                            }
                        }
                        self.renderPreview();
                    }
                });
            });

            return this;
        }
    });
});
